﻿/*-------------------------------------------------------------------------
 SASEBO_W with VCP and AIST OS Card interface.

 File name   : CipherModule/SASEBO_W_VCP_Card.cs
 Version     : 1.3
 Created     : APR/02/2012
 Last update : JUL/04/2012
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2012 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage.
 (http://www.aist.go.jp/aist_e/research_results/publications/synthesiology_e/vol3_no1/vol03_01_p86_p95.pdf)
 -------------------------------------------------------------------------*/

using System.Diagnostics;

namespace CipherModule
{
	//================================================================ SASEBO_W_VCP_Card
	public class SASEBO_W_VCP_Card : IBlockCipher
  {
		//************************************************ Variable
    RS232 vcp;


    //************************************************ Methods
    //------------------------------------------------ Constructor
    public SASEBO_W_VCP_Card(string port)
		{
      vcp = new RS232(port);
		}

    //------------------------------------------------ open()
    public void open()
    {
      vcp.open();
      vcp.flush();
    }

    //------------------------------------------------ close()
    public void close() { vcp.close(); }

    //------------------------------------------------ version()
    public string version() { return vcp.version(); }


    //************************************************ Interface : IBlockCipher
    //------------------------------------------------ setKey()
    public void setKey(byte[] key, int len)
    {
      int status = APDUSend(0x80, 0x12, 0x00, 0x00, key);
      Debug.WriteLine(status.ToString("X4"));
    }

		//------------------------------------------------ setEnc()
    public void setEnc() { /* nop */ }

		//------------------------------------------------ selDec()
    public void setDec() { /* nop */ }


		//------------------------------------------------ writeText()
    public void writeText (byte[] text, int len)
    {
      int status = APDUSend(0x80, 0x04, 0x04, 0x00, text);
      Debug.WriteLine(status.ToString("X4"));
    }

		//------------------------------------------------ readText()
    public void readText  (byte[] text, int len)
    {
      int status = APDURecv(0x80, 0xc0, 0x00, 0x00, text);
      Debug.WriteLine(status.ToString("X4"));
    }

		//------------------------------------------------ execute()
    public void execute () { /* nop */ }

    //------------------------------------------------ select()
		public void select(int cipher) { /* nop */ }

    //------------------------------------------------ setMode()
    public void setMode(int mode) { /* nop */ }


    //************************************************ APDU
    enum APDUHeaderIndex { CLA, INS, P1, P2, P3 };

    //------------------------------------------------ APDUSend()
    private int APDUSend(byte cla, byte ins, byte p1, byte p2, byte[] dat)
    {
      byte[] buf = new byte[16];

      // Send header
      byte[] header = new byte[5];
      header[(int)APDUHeaderIndex.CLA] = cla;
      header[(int)APDUHeaderIndex.INS] = ins;
      header[(int)APDUHeaderIndex.P1]  = p1;
      header[(int)APDUHeaderIndex.P2]  = p2;
      header[(int)APDUHeaderIndex.P3]  = (byte)((dat==null)? 0 : dat.Length);
      vcp.write(header, header.Length);

      // Ack
      vcp.read(buf, 1);
      if (buf[0] != header[(int)APDUHeaderIndex.INS]) { return -1; }

      // Send data
      if (dat != null) vcp.write(dat, dat.Length);
      vcp.read(buf, 2);
      return ((int)buf[0] << 8) + (int)buf[1];
    }

    //------------------------------------------------ APDURecv()
    private int APDURecv(byte cla, byte ins, byte p1, byte p2, byte[] dat)
    {
      byte[] buf = new byte[16];

      // Send header
      byte[] header = new byte[5];
      header[(int)APDUHeaderIndex.CLA] = cla;
      header[(int)APDUHeaderIndex.INS] = ins;
      header[(int)APDUHeaderIndex.P1]  = p1;
      header[(int)APDUHeaderIndex.P2]  = p2;
      header[(int)APDUHeaderIndex.P3]  = (byte)dat.Length;
      vcp.write(header, header.Length);

      // Ack
      vcp.read(buf, 1);
      if (buf[0] != header[(int)APDUHeaderIndex.INS]) { return -1; }

      // Receive data
      vcp.read(dat, dat.Length);
      vcp.read(buf, 2);
      return ((int)buf[0] << 8) + (int)buf[1];
    }

  }

}
